
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.LinearLayout
import io.skore.video360kaltura.R
import io.skore.video360kaltura.VideoPlayerController
import com.kaltura.playkit.player.PKHttpClientManager
import com.kaltura.tvplayer.KalturaOvpPlayer
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory

class VideoView internal constructor(private val context: Context?, viewId: Int, args: Any, binaryMessenger: BinaryMessenger) : PlatformView {
    private var player_view: FrameLayout? = null
    private var videoPlayerController = VideoPlayerController(context, object: VideoPlayerController.ViewCreatedListener {
        override fun onViewCreated(view: View) {
            player_view?.removeAllViews()
            player_view?.addView(view)
        }

        override fun changeViewSize(args: HashMap<*, *>) {
            updateViewSize(args)
        }
    }, viewId, binaryMessenger)

    private val innerView: View by lazy {
        LayoutInflater.from(context).inflate(R.layout.video_view, null).apply {
            player_view = findViewById(R.id.player_view)
            (args as? HashMap<*, *>)?.let {
                initializePlayer(it.get("partnerId") as? String, it.get("ovpServerUrl") as? String)
                updateViewSize(it)
            }
        }
    }

    private fun updateViewSize(args: java.util.HashMap<*, *>) {
        if (args.containsKey("width") && args["width"] is Double && args.containsKey("height") && args["height"] is Double) {
            player_view?.layoutParams = LinearLayout.LayoutParams((args["width"] as Double).toInt(), (args["height"] as Double).toInt())
        }
    }

    override fun getView(): View {
        return innerView
    }

    override fun dispose() {
        videoPlayerController.dispose()
    }

    override fun onFlutterViewDetached() {
        super.onFlutterViewDetached()
        videoPlayerController.dispose()
    }

    private fun initializePlayer(partnerId: String?, ovpServerUrl: String?) {
        if (partnerId?.toIntOrNull() == null || ovpServerUrl.isNullOrEmpty()) {
            return
        }

        KalturaOvpPlayer.initialize(context, partnerId.toInt(), ovpServerUrl)
        doConnectionsWarmup(ovpServerUrl)
        videoPlayerController.loadPlayer(partnerId.toInt(), ovpServerUrl)
    }

    private fun doConnectionsWarmup(ovpServerUrl: String) {
        PKHttpClientManager.setHttpProvider("okhttp")
        PKHttpClientManager.warmUp(ovpServerUrl)
    }
}

class VideoViewFactory(private val binaryMessenger: BinaryMessenger) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {
    override fun create(context: Context?, viewId: Int, args: Any): PlatformView {
        return VideoView(context, viewId, args, binaryMessenger)
    }
}
